////////////////////////////////////////////////////////////////////////////////
// auls_zbuffer.cpp
// Zobt@XNvgpDLL
////////////////////////////////////////////////////////////////////////////////

#include <aulslib/exedit.h>
#include <aulslib/lua.h>
#include <aulslib/memref.h>

using namespace auls;

CAMERA_ZBUFFER **gpp_zbuffer;

int l_dist(lua_State *L)
{
	if(!*gpp_zbuffer) return 0;
	// stack
	// 1 : ŋߋ
	// 2 : ŉ
	// 3-5 : ŋߐFAŉFA`F
	// 6 : ő摜
	// 7 - 9 : pixeldata, w, h
	int min_dist = lua_tointeger(L, 1);
	int max_dist = lua_tointeger(L, 2);
	ColorRGBA col_min_dist = (DWORD)lua_tonumber(L, 3);
	col_min_dist.a = 255;
	ColorRGBA col_max_dist = (DWORD)lua_tonumber(L, 4);
	col_max_dist.a = 255;
	ColorRGBA col_undraw;
	if(lua_type(L, 5) != LUA_TNIL) {
		col_undraw = (DWORD)lua_tonumber(L, 5);
		col_undraw.a = 255;
	} else col_undraw = 0;
	int image_max_w = lua_tointeger(L, 6) + 8;
	luaA_PixelData pd(L, 7);

	CAMERA_ZBUFFER *zbuf = *gpp_zbuffer;
	if(min_dist > max_dist) {
		Swap(min_dist, max_dist);
		Swap(col_min_dist, col_max_dist);
	}

	for(int y = 0; y < pd.height; y++) {
		for(int x = 0; x < pd.width; x++) {
			ColorRGBA col;
			CAMERA_ZBUFFER z = zbuf[x + y * image_max_w];
			if(!z.distance) {
				col = col_undraw;
			} else {
				int dist = z.GetDistance();
				if(dist <= min_dist) {
					col = col_min_dist;
				} else if(dist >= max_dist) {
					col = col_max_dist;
				} else {
					col.r = col_min_dist.r + (col_max_dist.r - col_min_dist.r) * (dist - min_dist) / (max_dist - min_dist);
					col.g = col_min_dist.g + (col_max_dist.g - col_min_dist.g) * (dist - min_dist) / (max_dist - min_dist);
					col.b = col_min_dist.b + (col_max_dist.b - col_min_dist.b) * (dist - min_dist) / (max_dist - min_dist);
					col.a = 255;
				}
			}
			pd.Put(x, y, col);
		}
	}

	return 3;
}

int l_rotate(lua_State *L)
{
	if(!*gpp_zbuffer) return 0;
	// stack
	// 1 : ŋߋ
	// 2 : ŉ
	// 3-5 : ŋߐFAŉFA`F
	// 6 : ő摜
	// 7 - 9 : pixeldata, w, h
	int min_rot = lua_tointeger(L, 1);
	int max_rot = lua_tointeger(L, 2);
	ColorRGBA col_min_rot = (DWORD)lua_tonumber(L, 3);
	col_min_rot.a = 255;
	ColorRGBA col_max_rot = (DWORD)lua_tonumber(L, 4);
	col_max_rot.a = 255;
	ColorRGBA col_undraw;
	if(lua_type(L, 5) != LUA_TNIL) {
		col_undraw = (DWORD)lua_tonumber(L, 5);
		col_undraw.a = 255;
	} else col_undraw = 0;
	int image_max_w = lua_tointeger(L, 6) + 8;
	luaA_PixelData pd(L, 7);

	CAMERA_ZBUFFER *zbuf = *gpp_zbuffer;
	if(min_rot > max_rot) {
		Swap(min_rot, max_rot);
		Swap(col_min_rot, col_max_rot);
	}

	for(int y = 0; y < pd.height; y++) {
		for(int x = 0; x < pd.width; x++) {
			ColorRGBA col;
			CAMERA_ZBUFFER z = zbuf[x + y * image_max_w];
			if(!z.distance) {
				col = col_undraw;
			} else {
				int rot = z.rotate;
				if(rot <= min_rot) {
					col = col_min_rot;
				} else if(rot >= max_rot) {
					col = col_max_rot;
				} else {
					col.r = col_min_rot.r + (col_max_rot.r - col_min_rot.r) * (rot - min_rot) / (max_rot - min_rot);
					col.g = col_min_rot.g + (col_max_rot.g - col_min_rot.g) * (rot - min_rot) / (max_rot - min_rot);
					col.b = col_min_rot.b + (col_max_rot.b - col_min_rot.b) * (rot - min_rot) / (max_rot - min_rot);
					col.a = 255;
				}
			}
			pd.Put(x, y, col);

		}
	}

	return 3;
}

#define R(name) { #name, l_##name }
luaL_reg table[] = {
	R(dist),
	R(rotate),
	{ NULL, NULL },
};
#undef R

extern "C" __declspec(dllexport) int luaopen_auls_zbuffer(lua_State *L)
{
	CMemref memref;
	if(!memref.Init(NULL)) goto failure;
	gpp_zbuffer = memref.Exedit_CameraZBuffer();
	if(!gpp_zbuffer) goto failure;
	auls::luaA_register(L, "auls", "zbuffer", table);
	return 1;

failure:
	OutputDebugString(memref.GetErrorString());
	return 0;
}
